/*
 * Copyright (c) 2011, Code Aurora Forum. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
/***********************************************************************/
/* Modified by                                                         */
/* (C) NEC CASIO Mobile Communications, Ltd. 2013                      */
/***********************************************************************/


/*
 * Qualcomm PMIC PM8xxx Thermal Manager driver
 */

#ifndef __PM8XXX_TM_H
#define __PM8XXX_TM_H

#include <linux/errno.h>
#include <linux/thermal.h>

#define PM8XXX_TM_DEV_NAME	"pm8xxx-tm"

enum pm8xxx_tm_adc_type {
	PM8XXX_TM_ADC_NONE,	/* Estimates temp based on overload level. */
	PM8XXX_TM_ADC_PM8058_ADC,
	PM8XXX_TM_ADC_PM8XXX_ADC,
};

struct pm8xxx_tm_core_data {
	int				adc_channel;
	unsigned long			default_no_adc_temp;
	enum pm8xxx_tm_adc_type		adc_type;
	u16				reg_addr_temp_alarm_ctrl;
	u16				reg_addr_temp_alarm_pwm;
	char				*tm_name;
	char				*irq_name_temp_stat;
	char				*irq_name_over_temp;
};

#define SENSOR_THERM_ABNORMAL_VAL   0xa999      /* -99.9 */

typedef struct
{
    u16    ad;
    u16    disp;
} therm_table_type;

static const therm_table_type Temperature_Table[] ={
{0x7FFF,SENSOR_THERM_ABNORMAL_VAL},
{0x3B5B,SENSOR_THERM_ABNORMAL_VAL},
{0x3B5A,0xa300},        /* -30.0 */
{0x3B4E,0xa299},        /* -29.9 */
{0x3B41,0xa298},        /* -29.8 */
{0x3B35,0xa297},        /* -29.7 */
{0x3B28,0xa296},        /* -29.6 */
{0x3B1B,0xa295},        /* -29.5 */
{0x3B0E,0xa294},        /* -29.4 */
{0x3B01,0xa293},        /* -29.3 */
{0x3AF4,0xa292},        /* -29.2 */
{0x3AE6,0xa291},        /* -29.1 */
{0x3AD9,0xa290},        /* -29.0 */
{0x3ACC,0xa289},        /* -28.9 */
{0x3ABF,0xa288},        /* -28.8 */
{0x3AB2,0xa287},        /* -28.7 */
{0x3AA5,0xa286},        /* -28.6 */
{0x3A98,0xa285},        /* -28.5 */
{0x3A8B,0xa284},        /* -28.4 */
{0x3A7D,0xa283},        /* -28.3 */
{0x3A70,0xa282},        /* -28.2 */
{0x3A62,0xa281},        /* -28.1 */
{0x3A54,0xa280},        /* -28.0 */
{0x3A47,0xa279},        /* -27.9 */
{0x3A3A,0xa278},        /* -27.8 */
{0x3A2D,0xa277},        /* -27.7 */
{0x3A1F,0xa276},        /* -27.6 */
{0x3A12,0xa275},        /* -27.5 */
{0x3A04,0xa274},        /* -27.4 */
{0x39F6,0xa273},        /* -27.3 */
{0x39E8,0xa272},        /* -27.2 */
{0x39DA,0xa271},        /* -27.1 */
{0x39CC,0xa270},        /* -27.0 */
{0x39BE,0xa269},        /* -26.9 */
{0x39B1,0xa268},        /* -26.8 */
{0x39A3,0xa267},        /* -26.7 */
{0x3995,0xa266},        /* -26.6 */
{0x3987,0xa265},        /* -26.5 */
{0x3979,0xa264},        /* -26.4 */
{0x396B,0xa263},        /* -26.3 */
{0x395D,0xa262},        /* -26.2 */
{0x394E,0xa261},        /* -26.1 */
{0x3940,0xa260},        /* -26.0 */
{0x3932,0xa259},        /* -25.9 */
{0x3924,0xa258},        /* -25.8 */
{0x3916,0xa257},        /* -25.7 */
{0x3908,0xa256},        /* -25.6 */
{0x38F9,0xa255},        /* -25.5 */
{0x38EB,0xa254},        /* -25.4 */
{0x38DC,0xa253},        /* -25.3 */
{0x38CE,0xa252},        /* -25.2 */
{0x38BF,0xa251},        /* -25.1 */
{0x38B0,0xa250},        /* -25.0 */
{0x38A2,0xa249},        /* -24.9 */
{0x3893,0xa248},        /* -24.8 */
{0x3885,0xa247},        /* -24.7 */
{0x3876,0xa246},        /* -24.6 */
{0x3868,0xa245},        /* -24.5 */
{0x3859,0xa244},        /* -24.4 */
{0x384A,0xa243},        /* -24.3 */
{0x383B,0xa242},        /* -24.2 */
{0x382C,0xa241},        /* -24.1 */
{0x381D,0xa240},        /* -24.0 */
{0x380E,0xa239},        /* -23.9 */
{0x37FF,0xa238},        /* -23.8 */
{0x37F1,0xa237},        /* -23.7 */
{0x37E2,0xa236},        /* -23.6 */
{0x37D3,0xa235},        /* -23.5 */
{0x37C3,0xa234},        /* -23.4 */
{0x37B4,0xa233},        /* -23.3 */
{0x37A5,0xa232},        /* -23.2 */
{0x3795,0xa231},        /* -23.1 */
{0x3785,0xa230},        /* -23.0 */
{0x3776,0xa229},        /* -22.9 */
{0x3767,0xa228},        /* -22.8 */
{0x3758,0xa227},        /* -22.7 */
{0x3749,0xa226},        /* -22.6 */
{0x373A,0xa225},        /* -22.5 */
{0x372A,0xa224},        /* -22.4 */
{0x371B,0xa223},        /* -22.3 */
{0x370B,0xa222},        /* -22.2 */
{0x36FB,0xa221},        /* -22.1 */
{0x36EB,0xa220},        /* -22.0 */
{0x36DC,0xa219},        /* -21.9 */
{0x36CC,0xa218},        /* -21.8 */
{0x36BD,0xa217},        /* -21.7 */
{0x36AD,0xa216},        /* -21.6 */
{0x369E,0xa215},        /* -21.5 */
{0x368E,0xa214},        /* -21.4 */
{0x367E,0xa213},        /* -21.3 */
{0x366E,0xa212},        /* -21.2 */
{0x365D,0xa211},        /* -21.1 */
{0x364D,0xa210},        /* -21.0 */
{0x363D,0xa209},        /* -20.9 */
{0x362E,0xa208},        /* -20.8 */
{0x361E,0xa207},        /* -20.7 */
{0x360E,0xa206},        /* -20.6 */
{0x35FE,0xa205},        /* -20.5 */
{0x35EE,0xa204},        /* -20.4 */
{0x35DD,0xa203},        /* -20.3 */
{0x35CD,0xa202},        /* -20.2 */
{0x35BC,0xa201},        /* -20.1 */
{0x35AC,0xa200},        /* -20.0 */
{0x359C,0xa199},        /* -19.9 */
{0x358C,0xa198},        /* -19.8 */
{0x357C,0xa197},        /* -19.7 */
{0x356B,0xa196},        /* -19.6 */
{0x355B,0xa195},        /* -19.5 */
{0x354B,0xa194},        /* -19.4 */
{0x353A,0xa193},        /* -19.3 */
{0x3529,0xa192},        /* -19.2 */
{0x3518,0xa191},        /* -19.1 */
{0x3507,0xa190},        /* -19.0 */
{0x34F7,0xa189},        /* -18.9 */
{0x34E7,0xa188},        /* -18.8 */
{0x34D6,0xa187},        /* -18.7 */
{0x34C6,0xa186},        /* -18.6 */
{0x34B5,0xa185},        /* -18.5 */
{0x34A4,0xa184},        /* -18.4 */
{0x3493,0xa183},        /* -18.3 */
{0x3482,0xa182},        /* -18.2 */
{0x3471,0xa181},        /* -18.1 */
{0x3460,0xa180},        /* -18.0 */
{0x344F,0xa179},        /* -17.9 */
{0x343F,0xa178},        /* -17.8 */
{0x342E,0xa177},        /* -17.7 */
{0x341D,0xa176},        /* -17.6 */
{0x340C,0xa175},        /* -17.5 */
{0x33FB,0xa174},        /* -17.4 */
{0x33EA,0xa173},        /* -17.3 */
{0x33D9,0xa172},        /* -17.2 */
{0x33C7,0xa171},        /* -17.1 */
{0x33B6,0xa170},        /* -17.0 */
{0x33A5,0xa169},        /* -16.9 */
{0x3394,0xa168},        /* -16.8 */
{0x3383,0xa167},        /* -16.7 */
{0x3372,0xa166},        /* -16.6 */
{0x3360,0xa165},        /* -16.5 */
{0x334F,0xa164},        /* -16.4 */
{0x333E,0xa163},        /* -16.3 */
{0x332C,0xa162},        /* -16.2 */
{0x331A,0xa161},        /* -16.1 */
{0x3308,0xa160},        /* -16.0 */
{0x32F7,0xa159},        /* -15.9 */
{0x32E6,0xa158},        /* -15.8 */
{0x32D5,0xa157},        /* -15.7 */
{0x32C3,0xa156},        /* -15.6 */
{0x32B2,0xa155},        /* -15.5 */
{0x32A0,0xa154},        /* -15.4 */
{0x328E,0xa153},        /* -15.3 */
{0x327D,0xa152},        /* -15.2 */
{0x326A,0xa151},        /* -15.1 */
{0x3258,0xa150},        /* -15.0 */
{0x3247,0xa149},        /* -14.9 */
{0x3236,0xa148},        /* -14.8 */
{0x3224,0xa147},        /* -14.7 */
{0x3212,0xa146},        /* -14.6 */
{0x3201,0xa145},        /* -14.5 */
{0x31EF,0xa144},        /* -14.4 */
{0x31DD,0xa143},        /* -14.3 */
{0x31CB,0xa142},        /* -14.2 */
{0x31B8,0xa141},        /* -14.1 */
{0x31A6,0xa140},        /* -14.0 */
{0x3194,0xa139},        /* -13.9 */
{0x3183,0xa138},        /* -13.8 */
{0x3171,0xa137},        /* -13.7 */
{0x315F,0xa136},        /* -13.6 */
{0x314D,0xa135},        /* -13.5 */
{0x313B,0xa134},        /* -13.4 */
{0x3129,0xa133},        /* -13.3 */
{0x3116,0xa132},        /* -13.2 */
{0x3104,0xa131},        /* -13.1 */
{0x30F1,0xa130},        /* -13.0 */
{0x30DF,0xa129},        /* -12.9 */
{0x30CD,0xa128},        /* -12.8 */
{0x30BB,0xa127},        /* -12.7 */
{0x30A9,0xa126},        /* -12.6 */
{0x3097,0xa125},        /* -12.5 */
{0x3085,0xa124},        /* -12.4 */
{0x3072,0xa123},        /* -12.3 */
{0x305F,0xa122},        /* -12.2 */
{0x304D,0xa121},        /* -12.1 */
{0x303A,0xa120},        /* -12.0 */
{0x3028,0xa119},        /* -11.9 */
{0x3016,0xa118},        /* -11.8 */
{0x3003,0xa117},        /* -11.7 */
{0x2FF1,0xa116},        /* -11.6 */
{0x2FDF,0xa115},        /* -11.5 */
{0x2FCC,0xa114},        /* -11.4 */
{0x2FB9,0xa113},        /* -11.3 */
{0x2FA7,0xa112},        /* -11.2 */
{0x2F94,0xa111},        /* -11.1 */
{0x2F81,0xa110},        /* -11.0 */
{0x2F6E,0xa109},        /* -10.9 */
{0x2F5C,0xa108},        /* -10.8 */
{0x2F4A,0xa107},        /* -10.7 */
{0x2F37,0xa106},        /* -10.6 */
{0x2F24,0xa105},        /* -10.5 */
{0x2F12,0xa104},        /* -10.4 */
{0x2EFF,0xa103},        /* -10.3 */
{0x2EEC,0xa102},        /* -10.2 */
{0x2ED9,0xa101},        /* -10.1 */
{0x2EC5,0xa100},        /* -10.0 */
{0x2EB3,0xa099},        /* - 9.9 */
{0x2EA1,0xa098},        /* - 9.8 */
{0x2E8E,0xa097},        /* - 9.7 */
{0x2E7B,0xa096},        /* - 9.6 */
{0x2E68,0xa095},        /* - 9.5 */
{0x2E56,0xa094},        /* - 9.4 */
{0x2E43,0xa093},        /* - 9.3 */
{0x2E2F,0xa092},        /* - 9.2 */
{0x2E1C,0xa091},        /* - 9.1 */
{0x2E09,0xa090},        /* - 9.0 */
{0x2DF6,0xa089},        /* - 8.9 */
{0x2DE4,0xa088},        /* - 8.8 */
{0x2DD1,0xa087},        /* - 8.7 */
{0x2DBE,0xa086},        /* - 8.6 */
{0x2DAB,0xa085},        /* - 8.5 */
{0x2D98,0xa084},        /* - 8.4 */
{0x2D85,0xa083},        /* - 8.3 */
{0x2D71,0xa082},        /* - 8.2 */
{0x2D5E,0xa081},        /* - 8.1 */
{0x2D4A,0xa080},        /* - 8.0 */
{0x2D38,0xa079},        /* - 7.9 */
{0x2D25,0xa078},        /* - 7.8 */
{0x2D12,0xa077},        /* - 7.7 */
{0x2CFF,0xa076},        /* - 7.6 */
{0x2CEC,0xa075},        /* - 7.5 */
{0x2CD9,0xa074},        /* - 7.4 */
{0x2CC5,0xa073},        /* - 7.3 */
{0x2CB2,0xa072},        /* - 7.2 */
{0x2C9E,0xa071},        /* - 7.1 */
{0x2C8B,0xa070},        /* - 7.0 */
{0x2C78,0xa069},        /* - 6.9 */
{0x2C65,0xa068},        /* - 6.8 */
{0x2C52,0xa067},        /* - 6.7 */
{0x2C3F,0xa066},        /* - 6.6 */
{0x2C2C,0xa065},        /* - 6.5 */
{0x2C18,0xa064},        /* - 6.4 */
{0x2C05,0xa063},        /* - 6.3 */
{0x2BF1,0xa062},        /* - 6.2 */
{0x2BDE,0xa061},        /* - 6.1 */
{0x2BCA,0xa060},        /* - 6.0 */
{0x2BB7,0xa059},        /* - 5.9 */
{0x2BA4,0xa058},        /* - 5.8 */
{0x2B91,0xa057},        /* - 5.7 */
{0x2B7E,0xa056},        /* - 5.6 */
{0x2B6B,0xa055},        /* - 5.5 */
{0x2B57,0xa054},        /* - 5.4 */
{0x2B44,0xa053},        /* - 5.3 */
{0x2B30,0xa052},        /* - 5.2 */
{0x2B1D,0xa051},        /* - 5.1 */
{0x2B09,0xa050},        /* - 5.0 */
{0x2AF6,0xa049},        /* - 4.9 */
{0x2AE3,0xa048},        /* - 4.8 */
{0x2ACF,0xa047},        /* - 4.7 */
{0x2ABC,0xa046},        /* - 4.6 */
{0x2AA9,0xa045},        /* - 4.5 */
{0x2A95,0xa044},        /* - 4.4 */
{0x2A81,0xa043},        /* - 4.3 */
{0x2A6E,0xa042},        /* - 4.2 */
{0x2A5A,0xa041},        /* - 4.1 */
{0x2A46,0xa040},        /* - 4.0 */
{0x2A33,0xa039},        /* - 3.9 */
{0x2A20,0xa038},        /* - 3.8 */
{0x2A0D,0xa037},        /* - 3.7 */
{0x29F9,0xa036},        /* - 3.6 */
{0x29E6,0xa035},        /* - 3.5 */
{0x29D2,0xa034},        /* - 3.4 */
{0x29BF,0xa033},        /* - 3.3 */
{0x29AB,0xa032},        /* - 3.2 */
{0x2997,0xa031},        /* - 3.1 */
{0x2983,0xa030},        /* - 3.0 */
{0x2970,0xa029},        /* - 2.9 */
{0x295D,0xa028},        /* - 2.8 */
{0x294A,0xa027},        /* - 2.7 */
{0x2936,0xa026},        /* - 2.6 */
{0x2923,0xa025},        /* - 2.5 */
{0x290F,0xa024},        /* - 2.4 */
{0x28FC,0xa023},        /* - 2.3 */
{0x28E8,0xa022},        /* - 2.2 */
{0x28D4,0xa021},        /* - 2.1 */
{0x28C0,0xa020},        /* - 2.0 */
{0x28AD,0xa019},        /* - 1.9 */
{0x289A,0xa018},        /* - 1.8 */
{0x2886,0xa017},        /* - 1.7 */
{0x2873,0xa016},        /* - 1.6 */
{0x285F,0xa015},        /* - 1.5 */
{0x284C,0xa014},        /* - 1.4 */
{0x2838,0xa013},        /* - 1.3 */
{0x2824,0xa012},        /* - 1.2 */
{0x2810,0xa011},        /* - 1.1 */
{0x27FC,0xa010},        /* - 1.0 */
{0x27E9,0xa009},        /* - 0.9 */
{0x27D6,0xa008},        /* - 0.8 */
{0x27C3,0xa007},        /* - 0.7 */
{0x27AF,0xa006},        /* - 0.6 */
{0x279C,0xa005},        /* - 0.5 */
{0x2788,0xa004},        /* - 0.4 */
{0x2775,0xa003},        /* - 0.3 */
{0x2761,0xa002},        /* - 0.2 */
{0x274D,0xa001},        /* - 0.1 */
{0x2739,0x0000},        /*   0.0 */
{0x2726,0x0001},        /*   0.1 */
{0x2713,0x0002},        /*   0.2 */
{0x26FF,0x0003},        /*   0.3 */
{0x26EC,0x0004},        /*   0.4 */
{0x26D8,0x0005},        /*   0.5 */
{0x26C5,0x0006},        /*   0.6 */
{0x26B1,0x0007},        /*   0.7 */
{0x269D,0x0008},        /*   0.8 */
{0x2689,0x0009},        /*   0.9 */
{0x2675,0x0010},        /*   1.0 */
{0x2662,0x0011},        /*   1.1 */
{0x264F,0x0012},        /*   1.2 */
{0x263C,0x0013},        /*   1.3 */
{0x2629,0x0014},        /*   1.4 */
{0x2615,0x0015},        /*   1.5 */
{0x2602,0x0016},        /*   1.6 */
{0x25EE,0x0017},        /*   1.7 */
{0x25DB,0x0018},        /*   1.8 */
{0x25C7,0x0019},        /*   1.9 */
{0x25B3,0x0020},        /*   2.0 */
{0x25A0,0x0021},        /*   2.1 */
{0x258D,0x0022},        /*   2.2 */
{0x2579,0x0023},        /*   2.3 */
{0x2566,0x0024},        /*   2.4 */
{0x2553,0x0025},        /*   2.5 */
{0x253F,0x0026},        /*   2.6 */
{0x252B,0x0027},        /*   2.7 */
{0x2518,0x0028},        /*   2.8 */
{0x2504,0x0029},        /*   2.9 */
{0x24F0,0x0030},        /*   3.0 */
{0x24DD,0x0031},        /*   3.1 */
{0x24CA,0x0032},        /*   3.2 */
{0x24B7,0x0033},        /*   3.3 */
{0x24A4,0x0034},        /*   3.4 */
{0x2490,0x0035},        /*   3.5 */
{0x247D,0x0036},        /*   3.6 */
{0x246A,0x0037},        /*   3.7 */
{0x2456,0x0038},        /*   3.8 */
{0x2443,0x0039},        /*   3.9 */
{0x242F,0x0040},        /*   4.0 */
{0x241C,0x0041},        /*   4.1 */
{0x2409,0x0042},        /*   4.2 */
{0x23F6,0x0043},        /*   4.3 */
{0x23E3,0x0044},        /*   4.4 */
{0x23D0,0x0045},        /*   4.5 */
{0x23BC,0x0046},        /*   4.6 */
{0x23A9,0x0047},        /*   4.7 */
{0x2396,0x0048},        /*   4.8 */
{0x2382,0x0049},        /*   4.9 */
{0x236E,0x0050},        /*   5.0 */
{0x235C,0x0051},        /*   5.1 */
{0x2349,0x0052},        /*   5.2 */
{0x2336,0x0053},        /*   5.3 */
{0x2323,0x0054},        /*   5.4 */
{0x2310,0x0055},        /*   5.5 */
{0x22FC,0x0056},        /*   5.6 */
{0x22E9,0x0057},        /*   5.7 */
{0x22D6,0x0058},        /*   5.8 */
{0x22C2,0x0059},        /*   5.9 */
{0x22AF,0x0060},        /*   6.0 */
{0x229C,0x0061},        /*   6.1 */
{0x2289,0x0062},        /*   6.2 */
{0x2276,0x0063},        /*   6.3 */
{0x2263,0x0064},        /*   6.4 */
{0x2250,0x0065},        /*   6.5 */
{0x223D,0x0066},        /*   6.6 */
{0x222A,0x0067},        /*   6.7 */
{0x2216,0x0068},        /*   6.8 */
{0x2203,0x0069},        /*   6.9 */
{0x21EF,0x0070},        /*   7.0 */
{0x21DD,0x0071},        /*   7.1 */
{0x21CA,0x0072},        /*   7.2 */
{0x21B8,0x0073},        /*   7.3 */
{0x21A5,0x0074},        /*   7.4 */
{0x2192,0x0075},        /*   7.5 */
{0x217F,0x0076},        /*   7.6 */
{0x216C,0x0077},        /*   7.7 */
{0x2159,0x0078},        /*   7.8 */
{0x2146,0x0079},        /*   7.9 */
{0x2132,0x0080},        /*   8.0 */
{0x2120,0x0081},        /*   8.1 */
{0x210E,0x0082},        /*   8.2 */
{0x20FB,0x0083},        /*   8.3 */
{0x20E8,0x0084},        /*   8.4 */
{0x20D6,0x0085},        /*   8.5 */
{0x20C3,0x0086},        /*   8.6 */
{0x20B0,0x0087},        /*   8.7 */
{0x209D,0x0088},        /*   8.8 */
{0x208A,0x0089},        /*   8.9 */
{0x2077,0x0090},        /*   9.0 */
{0x2065,0x0091},        /*   9.1 */
{0x2052,0x0092},        /*   9.2 */
{0x2040,0x0093},        /*   9.3 */
{0x202E,0x0094},        /*   9.4 */
{0x201B,0x0095},        /*   9.5 */
{0x2008,0x0096},        /*   9.6 */
{0x1FF6,0x0097},        /*   9.7 */
{0x1FE3,0x0098},        /*   9.8 */
{0x1FD0,0x0099},        /*   9.9 */
{0x1FBD,0x0100},        /*  10.0 */
{0x1FAB,0x0101},        /*  10.1 */
{0x1F99,0x0102},        /*  10.2 */
{0x1F87,0x0103},        /*  10.3 */
{0x1F75,0x0104},        /*  10.4 */
{0x1F62,0x0105},        /*  10.5 */
{0x1F50,0x0106},        /*  10.6 */
{0x1F3E,0x0107},        /*  10.7 */
{0x1F2B,0x0108},        /*  10.8 */
{0x1F19,0x0109},        /*  10.9 */
{0x1F06,0x0110},        /*  11.0 */
{0x1EF4,0x0111},        /*  11.1 */
{0x1EE2,0x0112},        /*  11.2 */
{0x1ED0,0x0113},        /*  11.3 */
{0x1EBD,0x0114},        /*  11.4 */
{0x1EAB,0x0115},        /*  11.5 */
{0x1E99,0x0116},        /*  11.6 */
{0x1E86,0x0117},        /*  11.7 */
{0x1E74,0x0118},        /*  11.8 */
{0x1E61,0x0119},        /*  11.9 */
{0x1E4F,0x0120},        /*  12.0 */
{0x1E3D,0x0121},        /*  12.1 */
{0x1E2C,0x0122},        /*  12.2 */
{0x1E1A,0x0123},        /*  12.3 */
{0x1E08,0x0124},        /*  12.4 */
{0x1DF7,0x0125},        /*  12.5 */
{0x1DE5,0x0126},        /*  12.6 */
{0x1DD3,0x0127},        /*  12.7 */
{0x1DC1,0x0128},        /*  12.8 */
{0x1DAF,0x0129},        /*  12.9 */
{0x1D9D,0x0130},        /*  13.0 */
{0x1D8B,0x0131},        /*  13.1 */
{0x1D7A,0x0132},        /*  13.2 */
{0x1D68,0x0133},        /*  13.3 */
{0x1D56,0x0134},        /*  13.4 */
{0x1D45,0x0135},        /*  13.5 */
{0x1D33,0x0136},        /*  13.6 */
{0x1D21,0x0137},        /*  13.7 */
{0x1D0F,0x0138},        /*  13.8 */
{0x1CFD,0x0139},        /*  13.9 */
{0x1CEB,0x0140},        /*  14.0 */
{0x1CDA,0x0141},        /*  14.1 */
{0x1CC8,0x0142},        /*  14.2 */
{0x1CB7,0x0143},        /*  14.3 */
{0x1CA5,0x0144},        /*  14.4 */
{0x1C94,0x0145},        /*  14.5 */
{0x1C82,0x0146},        /*  14.6 */
{0x1C71,0x0147},        /*  14.7 */
{0x1C5F,0x0148},        /*  14.8 */
{0x1C4D,0x0149},        /*  14.9 */
{0x1C3B,0x0150},        /*  15.0 */
{0x1C2A,0x0151},        /*  15.1 */
{0x1C19,0x0152},        /*  15.2 */
{0x1C08,0x0153},        /*  15.3 */
{0x1BF7,0x0154},        /*  15.4 */
{0x1BE6,0x0155},        /*  15.5 */
{0x1BD5,0x0156},        /*  15.6 */
{0x1BC3,0x0157},        /*  15.7 */
{0x1BB2,0x0158},        /*  15.8 */
{0x1BA1,0x0159},        /*  15.9 */
{0x1B8F,0x0160},        /*  16.0 */
{0x1B7E,0x0161},        /*  16.1 */
{0x1B6D,0x0162},        /*  16.2 */
{0x1B5C,0x0163},        /*  16.3 */
{0x1B4B,0x0164},        /*  16.4 */
{0x1B3A,0x0165},        /*  16.5 */
{0x1B29,0x0166},        /*  16.6 */
{0x1B18,0x0167},        /*  16.7 */
{0x1B06,0x0168},        /*  16.8 */
{0x1AF5,0x0169},        /*  16.9 */
{0x1AE4,0x0170},        /*  17.0 */
{0x1AD3,0x0171},        /*  17.1 */
{0x1AC3,0x0172},        /*  17.2 */
{0x1AB2,0x0173},        /*  17.3 */
{0x1AA1,0x0174},        /*  17.4 */
{0x1A91,0x0175},        /*  17.5 */
{0x1A80,0x0176},        /*  17.6 */
{0x1A6F,0x0177},        /*  17.7 */
{0x1A5E,0x0178},        /*  17.8 */
{0x1A4D,0x0179},        /*  17.9 */
{0x1A3C,0x0180},        /*  18.0 */
{0x1A2C,0x0181},        /*  18.1 */
{0x1A1C,0x0182},        /*  18.2 */
{0x1A0B,0x0183},        /*  18.3 */
{0x19FB,0x0184},        /*  18.4 */
{0x19EA,0x0185},        /*  18.5 */
{0x19DA,0x0186},        /*  18.6 */
{0x19C9,0x0187},        /*  18.7 */
{0x19B8,0x0188},        /*  18.8 */
{0x19A8,0x0189},        /*  18.9 */
{0x1997,0x0190},        /*  19.0 */
{0x1987,0x0191},        /*  19.1 */
{0x1977,0x0192},        /*  19.2 */
{0x1967,0x0193},        /*  19.3 */
{0x1957,0x0194},        /*  19.4 */
{0x1946,0x0195},        /*  19.5 */
{0x1936,0x0196},        /*  19.6 */
{0x1926,0x0197},        /*  19.7 */
{0x1916,0x0198},        /*  19.8 */
{0x1905,0x0199},        /*  19.9 */
{0x18F5,0x0200},        /*  20.0 */
{0x18E5,0x0201},        /*  20.1 */
{0x18D5,0x0202},        /*  20.2 */
{0x18C5,0x0203},        /*  20.3 */
{0x18B6,0x0204},        /*  20.4 */
{0x18A6,0x0205},        /*  20.5 */
{0x1896,0x0206},        /*  20.6 */
{0x1886,0x0207},        /*  20.7 */
{0x1876,0x0208},        /*  20.8 */
{0x1865,0x0209},        /*  20.9 */
{0x1855,0x0210},        /*  21.0 */
{0x1846,0x0211},        /*  21.1 */
{0x1836,0x0212},        /*  21.2 */
{0x1827,0x0213},        /*  21.3 */
{0x1817,0x0214},        /*  21.4 */
{0x1807,0x0215},        /*  21.5 */
{0x17F8,0x0216},        /*  21.6 */
{0x17E8,0x0217},        /*  21.7 */
{0x17D8,0x0218},        /*  21.8 */
{0x17C8,0x0219},        /*  21.9 */
{0x17B8,0x0220},        /*  22.0 */
{0x17A9,0x0221},        /*  22.1 */
{0x179A,0x0222},        /*  22.2 */
{0x178B,0x0223},        /*  22.3 */
{0x177C,0x0224},        /*  22.4 */
{0x176C,0x0225},        /*  22.5 */
{0x175D,0x0226},        /*  22.6 */
{0x174D,0x0227},        /*  22.7 */
{0x173E,0x0228},        /*  22.8 */
{0x172E,0x0229},        /*  22.9 */
{0x171F,0x0230},        /*  23.0 */
{0x1710,0x0231},        /*  23.1 */
{0x1701,0x0232},        /*  23.2 */
{0x16F2,0x0233},        /*  23.3 */
{0x16E3,0x0234},        /*  23.4 */
{0x16D4,0x0235},        /*  23.5 */
{0x16C5,0x0236},        /*  23.6 */
{0x16B6,0x0237},        /*  23.7 */
{0x16A7,0x0238},        /*  23.8 */
{0x1698,0x0239},        /*  23.9 */
{0x1689,0x0240},        /*  24.0 */
{0x167A,0x0241},        /*  24.1 */
{0x166C,0x0242},        /*  24.2 */
{0x165D,0x0243},        /*  24.3 */
{0x164E,0x0244},        /*  24.4 */
{0x163F,0x0245},        /*  24.5 */
{0x1631,0x0246},        /*  24.6 */
{0x1622,0x0247},        /*  24.7 */
{0x1613,0x0248},        /*  24.8 */
{0x1604,0x0249},        /*  24.9 */
{0x15F5,0x0250},        /*  25.0 */
{0x15E7,0x0251},        /*  25.1 */
{0x15D8,0x0252},        /*  25.2 */
{0x15CA,0x0253},        /*  25.3 */
{0x15BC,0x0254},        /*  25.4 */
{0x15AD,0x0255},        /*  25.5 */
{0x159F,0x0256},        /*  25.6 */
{0x1590,0x0257},        /*  25.7 */
{0x1582,0x0258},        /*  25.8 */
{0x1573,0x0259},        /*  25.9 */
{0x1564,0x0260},        /*  26.0 */
{0x1556,0x0261},        /*  26.1 */
{0x1548,0x0262},        /*  26.2 */
{0x153A,0x0263},        /*  26.3 */
{0x152C,0x0264},        /*  26.4 */
{0x151E,0x0265},        /*  26.5 */
{0x1510,0x0266},        /*  26.6 */
{0x1502,0x0267},        /*  26.7 */
{0x14F3,0x0268},        /*  26.8 */
{0x14E5,0x0269},        /*  26.9 */
{0x14D7,0x0270},        /*  27.0 */
{0x14C9,0x0271},        /*  27.1 */
{0x14BB,0x0272},        /*  27.2 */
{0x14AD,0x0273},        /*  27.3 */
{0x14A0,0x0274},        /*  27.4 */
{0x1492,0x0275},        /*  27.5 */
{0x1484,0x0276},        /*  27.6 */
{0x1476,0x0277},        /*  27.7 */
{0x1468,0x0278},        /*  27.8 */
{0x145A,0x0279},        /*  27.9 */
{0x144C,0x0280},        /*  28.0 */
{0x143F,0x0281},        /*  28.1 */
{0x1431,0x0282},        /*  28.2 */
{0x1424,0x0283},        /*  28.3 */
{0x1416,0x0284},        /*  28.4 */
{0x1408,0x0285},        /*  28.5 */
{0x13FB,0x0286},        /*  28.6 */
{0x13ED,0x0287},        /*  28.7 */
{0x13E0,0x0288},        /*  28.8 */
{0x13D2,0x0289},        /*  28.9 */
{0x13C4,0x0290},        /*  29.0 */
{0x13B7,0x0291},        /*  29.1 */
{0x13AA,0x0292},        /*  29.2 */
{0x139D,0x0293},        /*  29.3 */
{0x138F,0x0294},        /*  29.4 */
{0x1382,0x0295},        /*  29.5 */
{0x1375,0x0296},        /*  29.6 */
{0x1367,0x0297},        /*  29.7 */
{0x135A,0x0298},        /*  29.8 */
{0x134D,0x0299},        /*  29.9 */
{0x133F,0x0300},        /*  30.0 */
{0x1332,0x0301},        /*  30.1 */
{0x1325,0x0302},        /*  30.2 */
{0x1319,0x0303},        /*  30.3 */
{0x130C,0x0304},        /*  30.4 */
{0x12FF,0x0305},        /*  30.5 */
{0x12F2,0x0306},        /*  30.6 */
{0x12E5,0x0307},        /*  30.7 */
{0x12D8,0x0308},        /*  30.8 */
{0x12CB,0x0309},        /*  30.9 */
{0x12BE,0x0310},        /*  31.0 */
{0x12B1,0x0311},        /*  31.1 */
{0x12A4,0x0312},        /*  31.2 */
{0x1298,0x0313},        /*  31.3 */
{0x128B,0x0314},        /*  31.4 */
{0x127E,0x0315},        /*  31.5 */
{0x1272,0x0316},        /*  31.6 */
{0x1265,0x0317},        /*  31.7 */
{0x1258,0x0318},        /*  31.8 */
{0x124C,0x0319},        /*  31.9 */
{0x123F,0x0320},        /*  32.0 */
{0x1232,0x0321},        /*  32.1 */
{0x1226,0x0322},        /*  32.2 */
{0x121A,0x0323},        /*  32.3 */
{0x120D,0x0324},        /*  32.4 */
{0x1201,0x0325},        /*  32.5 */
{0x11F5,0x0326},        /*  32.6 */
{0x11E8,0x0327},        /*  32.7 */
{0x11DC,0x0328},        /*  32.8 */
{0x11CF,0x0329},        /*  32.9 */
{0x11C3,0x0330},        /*  33.0 */
{0x11B7,0x0331},        /*  33.1 */
{0x11AB,0x0332},        /*  33.2 */
{0x119F,0x0333},        /*  33.3 */
{0x1193,0x0334},        /*  33.4 */
{0x1186,0x0335},        /*  33.5 */
{0x117A,0x0336},        /*  33.6 */
{0x116E,0x0337},        /*  33.7 */
{0x1162,0x0338},        /*  33.8 */
{0x1156,0x0339},        /*  33.9 */
{0x114A,0x0340},        /*  34.0 */
{0x113E,0x0341},        /*  34.1 */
{0x1132,0x0342},        /*  34.2 */
{0x1126,0x0343},        /*  34.3 */
{0x111B,0x0344},        /*  34.4 */
{0x110F,0x0345},        /*  34.5 */
{0x1103,0x0346},        /*  34.6 */
{0x10F7,0x0347},        /*  34.7 */
{0x10EB,0x0348},        /*  34.8 */
{0x10DF,0x0349},        /*  34.9 */
{0x10D3,0x0350},        /*  35.0 */
{0x10C8,0x0351},        /*  35.1 */
{0x10BC,0x0352},        /*  35.2 */
{0x10B1,0x0353},        /*  35.3 */
{0x10A5,0x0354},        /*  35.4 */
{0x109A,0x0355},        /*  35.5 */
{0x108E,0x0356},        /*  35.6 */
{0x1083,0x0357},        /*  35.7 */
{0x1077,0x0358},        /*  35.8 */
{0x106B,0x0359},        /*  35.9 */
{0x1060,0x0360},        /*  36.0 */
{0x1055,0x0361},        /*  36.1 */
{0x104A,0x0362},        /*  36.2 */
{0x103E,0x0363},        /*  36.3 */
{0x1033,0x0364},        /*  36.4 */
{0x1028,0x0365},        /*  36.5 */
{0x101D,0x0366},        /*  36.6 */
{0x1011,0x0367},        /*  36.7 */
{0x1006,0x0368},        /*  36.8 */
{0x0FFB,0x0369},        /*  36.9 */
{0x0FEF,0x0370},        /*  37.0 */
{0x0FE5,0x0371},        /*  37.1 */
{0x0FDA,0x0372},        /*  37.2 */
{0x0FCF,0x0373},        /*  37.3 */
{0x0FC4,0x0374},        /*  37.4 */
{0x0FB9,0x0375},        /*  37.5 */
{0x0FAE,0x0376},        /*  37.6 */
{0x0FA3,0x0377},        /*  37.7 */
{0x0F98,0x0378},        /*  37.8 */
{0x0F8D,0x0379},        /*  37.9 */
{0x0F81,0x0380},        /*  38.0 */
{0x0F77,0x0381},        /*  38.1 */
{0x0F6C,0x0382},        /*  38.2 */
{0x0F61,0x0383},        /*  38.3 */
{0x0F57,0x0384},        /*  38.4 */
{0x0F4C,0x0385},        /*  38.5 */
{0x0F41,0x0386},        /*  38.6 */
{0x0F37,0x0387},        /*  38.7 */
{0x0F2C,0x0388},        /*  38.8 */
{0x0F21,0x0389},        /*  38.9 */
{0x0F16,0x0390},        /*  39.0 */
{0x0F0C,0x0391},        /*  39.1 */
{0x0F02,0x0392},        /*  39.2 */
{0x0EF7,0x0393},        /*  39.3 */
{0x0EED,0x0394},        /*  39.4 */
{0x0EE2,0x0395},        /*  39.5 */
{0x0ED8,0x0396},        /*  39.6 */
{0x0ECD,0x0397},        /*  39.7 */
{0x0EC3,0x0398},        /*  39.8 */
{0x0EB8,0x0399},        /*  39.9 */
{0x0EAE,0x0400},        /*  40.0 */
{0x0EA4,0x0401},        /*  40.1 */
{0x0E9A,0x0402},        /*  40.2 */
{0x0E8F,0x0403},        /*  40.3 */
{0x0E85,0x0404},        /*  40.4 */
{0x0E7B,0x0405},        /*  40.5 */
{0x0E71,0x0406},        /*  40.6 */
{0x0E67,0x0407},        /*  40.7 */
{0x0E5D,0x0408},        /*  40.8 */
{0x0E52,0x0409},        /*  40.9 */
{0x0E48,0x0410},        /*  41.0 */
{0x0E3E,0x0411},        /*  41.1 */
{0x0E34,0x0412},        /*  41.2 */
{0x0E2A,0x0413},        /*  41.3 */
{0x0E21,0x0414},        /*  41.4 */
{0x0E17,0x0415},        /*  41.5 */
{0x0E0D,0x0416},        /*  41.6 */
{0x0E03,0x0417},        /*  41.7 */
{0x0DF9,0x0418},        /*  41.8 */
{0x0DEF,0x0419},        /*  41.9 */
{0x0DE5,0x0420},        /*  42.0 */
{0x0DDB,0x0421},        /*  42.1 */
{0x0DD2,0x0422},        /*  42.2 */
{0x0DC8,0x0423},        /*  42.3 */
{0x0DBE,0x0424},        /*  42.4 */
{0x0DB5,0x0425},        /*  42.5 */
{0x0DAB,0x0426},        /*  42.6 */
{0x0DA1,0x0427},        /*  42.7 */
{0x0D98,0x0428},        /*  42.8 */
{0x0D8E,0x0429},        /*  42.9 */
{0x0D84,0x0430},        /*  43.0 */
{0x0D7B,0x0431},        /*  43.1 */
{0x0D71,0x0432},        /*  43.2 */
{0x0D68,0x0433},        /*  43.3 */
{0x0D5F,0x0434},        /*  43.4 */
{0x0D55,0x0435},        /*  43.5 */
{0x0D4C,0x0436},        /*  43.6 */
{0x0D42,0x0437},        /*  43.7 */
{0x0D39,0x0438},        /*  43.8 */
{0x0D2F,0x0439},        /*  43.9 */
{0x0D26,0x0440},        /*  44.0 */
{0x0D1D,0x0441},        /*  44.1 */
{0x0D14,0x0442},        /*  44.2 */
{0x0D0B,0x0443},        /*  44.3 */
{0x0D01,0x0444},        /*  44.4 */
{0x0CF8,0x0445},        /*  44.5 */
{0x0CEF,0x0446},        /*  44.6 */
{0x0CE6,0x0447},        /*  44.7 */
{0x0CDD,0x0448},        /*  44.8 */
{0x0CD3,0x0449},        /*  44.9 */
{0x0CCA,0x0450},        /*  45.0 */
{0x0CC1,0x0451},        /*  45.1 */
{0x0CB8,0x0452},        /*  45.2 */
{0x0CB0,0x0453},        /*  45.3 */
{0x0CA7,0x0454},        /*  45.4 */
{0x0C9E,0x0455},        /*  45.5 */
{0x0C95,0x0456},        /*  45.6 */
{0x0C8C,0x0457},        /*  45.7 */
{0x0C83,0x0458},        /*  45.8 */
{0x0C7A,0x0459},        /*  45.9 */
{0x0C71,0x0460},        /*  46.0 */
{0x0C68,0x0461},        /*  46.1 */
{0x0C60,0x0462},        /*  46.2 */
{0x0C57,0x0463},        /*  46.3 */
{0x0C4E,0x0464},        /*  46.4 */
{0x0C46,0x0465},        /*  46.5 */
{0x0C3D,0x0466},        /*  46.6 */
{0x0C34,0x0467},        /*  46.7 */
{0x0C2C,0x0468},        /*  46.8 */
{0x0C23,0x0469},        /*  46.9 */
{0x0C1A,0x0470},        /*  47.0 */
{0x0C12,0x0471},        /*  47.1 */
{0x0C09,0x0472},        /*  47.2 */
{0x0C01,0x0473},        /*  47.3 */
{0x0BF8,0x0474},        /*  47.4 */
{0x0BF0,0x0475},        /*  47.5 */
{0x0BE7,0x0476},        /*  47.6 */
{0x0BDF,0x0477},        /*  47.7 */
{0x0BD7,0x0478},        /*  47.8 */
{0x0BCE,0x0479},        /*  47.9 */
{0x0BC6,0x0480},        /*  48.0 */
{0x0BBD,0x0481},        /*  48.1 */
{0x0BB5,0x0482},        /*  48.2 */
{0x0BAD,0x0483},        /*  48.3 */
{0x0BA5,0x0484},        /*  48.4 */
{0x0B9C,0x0485},        /*  48.5 */
{0x0B94,0x0486},        /*  48.6 */
{0x0B8C,0x0487},        /*  48.7 */
{0x0B84,0x0488},        /*  48.8 */
{0x0B7B,0x0489},        /*  48.9 */
{0x0B73,0x0490},        /*  49.0 */
{0x0B6B,0x0491},        /*  49.1 */
{0x0B63,0x0492},        /*  49.2 */
{0x0B5B,0x0493},        /*  49.3 */
{0x0B53,0x0494},        /*  49.4 */
{0x0B4B,0x0495},        /*  49.5 */
{0x0B43,0x0496},        /*  49.6 */
{0x0B3B,0x0497},        /*  49.7 */
{0x0B33,0x0498},        /*  49.8 */
{0x0B2B,0x0499},        /*  49.9 */
{0x0B23,0x0500},        /*  50.0 */
{0x0B1B,0x0501},        /*  50.1 */
{0x0B13,0x0502},        /*  50.2 */
{0x0B0C,0x0503},        /*  50.3 */
{0x0B04,0x0504},        /*  50.4 */
{0x0AFC,0x0505},        /*  50.5 */
{0x0AF4,0x0506},        /*  50.6 */
{0x0AEC,0x0507},        /*  50.7 */
{0x0AE5,0x0508},        /*  50.8 */
{0x0ADD,0x0509},        /*  50.9 */
{0x0AD5,0x0510},        /*  51.0 */
{0x0ACD,0x0511},        /*  51.1 */
{0x0AC6,0x0512},        /*  51.2 */
{0x0ABE,0x0513},        /*  51.3 */
{0x0AB7,0x0514},        /*  51.4 */
{0x0AAF,0x0515},        /*  51.5 */
{0x0AA8,0x0516},        /*  51.6 */
{0x0AA0,0x0517},        /*  51.7 */
{0x0A98,0x0518},        /*  51.8 */
{0x0A91,0x0519},        /*  51.9 */
{0x0A89,0x0520},        /*  52.0 */
{0x0A82,0x0521},        /*  52.1 */
{0x0A7B,0x0522},        /*  52.2 */
{0x0A73,0x0523},        /*  52.3 */
{0x0A6C,0x0524},        /*  52.4 */
{0x0A65,0x0525},        /*  52.5 */
{0x0A5D,0x0526},        /*  52.6 */
{0x0A56,0x0527},        /*  52.7 */
{0x0A4E,0x0528},        /*  52.8 */
{0x0A47,0x0529},        /*  52.9 */
{0x0A40,0x0530},        /*  53.0 */
{0x0A38,0x0531},        /*  53.1 */
{0x0A31,0x0532},        /*  53.2 */
{0x0A2A,0x0533},        /*  53.3 */
{0x0A23,0x0534},        /*  53.4 */
{0x0A1C,0x0535},        /*  53.5 */
{0x0A15,0x0536},        /*  53.6 */
{0x0A0D,0x0537},        /*  53.7 */
{0x0A06,0x0538},        /*  53.8 */
{0x09FF,0x0539},        /*  53.9 */
{0x09F8,0x0540},        /*  54.0 */
{0x09F1,0x0541},        /*  54.1 */
{0x09EA,0x0542},        /*  54.2 */
{0x09E3,0x0543},        /*  54.3 */
{0x09DC,0x0544},        /*  54.4 */
{0x09D5,0x0545},        /*  54.5 */
{0x09CE,0x0546},        /*  54.6 */
{0x09C7,0x0547},        /*  54.7 */
{0x09C0,0x0548},        /*  54.8 */
{0x09B9,0x0549},        /*  54.9 */
{0x09B2,0x0550},        /*  55.0 */
{0x09AB,0x0551},        /*  55.1 */
{0x09A5,0x0552},        /*  55.2 */
{0x099E,0x0553},        /*  55.3 */
{0x0997,0x0554},        /*  55.4 */
{0x0990,0x0555},        /*  55.5 */
{0x098A,0x0556},        /*  55.6 */
{0x0983,0x0557},        /*  55.7 */
{0x097C,0x0558},        /*  55.8 */
{0x0975,0x0559},        /*  55.9 */
{0x096F,0x0560},        /*  56.0 */
{0x0968,0x0561},        /*  56.1 */
{0x0962,0x0562},        /*  56.2 */
{0x095B,0x0563},        /*  56.3 */
{0x0954,0x0564},        /*  56.4 */
{0x094E,0x0565},        /*  56.5 */
{0x0947,0x0566},        /*  56.6 */
{0x0941,0x0567},        /*  56.7 */
{0x093A,0x0568},        /*  56.8 */
{0x0933,0x0569},        /*  56.9 */
{0x092D,0x0570},        /*  57.0 */
{0x0926,0x0571},        /*  57.1 */
{0x0920,0x0572},        /*  57.2 */
{0x091A,0x0573},        /*  57.3 */
{0x0913,0x0574},        /*  57.4 */
{0x090D,0x0575},        /*  57.5 */
{0x0906,0x0576},        /*  57.6 */
{0x0900,0x0577},        /*  57.7 */
{0x08FA,0x0578},        /*  57.8 */
{0x08F3,0x0579},        /*  57.9 */
{0x08ED,0x0580},        /*  58.0 */
{0x08E6,0x0581},        /*  58.1 */
{0x08E0,0x0582},        /*  58.2 */
{0x08DA,0x0583},        /*  58.3 */
{0x08D4,0x0584},        /*  58.4 */
{0x08CE,0x0585},        /*  58.5 */
{0x08C8,0x0586},        /*  58.6 */
{0x08C1,0x0587},        /*  58.7 */
{0x08BB,0x0588},        /*  58.8 */
{0x08B5,0x0589},        /*  58.9 */
{0x08AF,0x0590},        /*  59.0 */
{0x08A9,0x0591},        /*  59.1 */
{0x08A3,0x0592},        /*  59.2 */
{0x089D,0x0593},        /*  59.3 */
{0x0897,0x0594},        /*  59.4 */
{0x0891,0x0595},        /*  59.5 */
{0x088A,0x0596},        /*  59.6 */
{0x0884,0x0597},        /*  59.7 */
{0x087E,0x0598},        /*  59.8 */
{0x0878,0x0599},        /*  59.9 */
{0x0872,0x0600},        /*  60.0 */
{0x0000,SENSOR_THERM_ABNORMAL_VAL}
};

int nc_pm8921_itemp_get_stage(u8 *stage);
int nc_pm8921_itemp_stage_override(enum thermal_device_mode mode);
u16 nc_pm8921_tm_get_temp(u16 therm_ad);
#endif
